'use strict'
var __extends =
  (this && this.__extends) ||
  (function () {
    var extendStatics = function (d, b) {
      extendStatics =
        Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array &&
          function (d, b) {
            d.__proto__ = b
          }) ||
        function (d, b) {
          for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]
        }
      return extendStatics(d, b)
    }
    return function (d, b) {
      if (typeof b !== 'function' && b !== null)
        throw new TypeError('Class extends value ' + String(b) + ' is not a constructor or null')
      extendStatics(d, b)
      function __() {
        this.constructor = d
      }
      d.prototype = b === null ? Object.create(b) : ((__.prototype = b.prototype), new __())
    }
  })()
var __assign =
  (this && this.__assign) ||
  function () {
    __assign =
      Object.assign ||
      function (t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
          s = arguments[i]
          for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p]
        }
        return t
      }
    return __assign.apply(this, arguments)
  }
Object.defineProperty(exports, '__esModule', { value: true })
exports.getFormatedParameters =
  exports.removeUndefinedFields =
  exports.Required =
  exports.extractQueryParameters =
  exports.extractParameter =
  exports.Cookie =
  exports.Header =
  exports.Path =
  exports.Query =
  exports.Resp =
  exports.Body =
  exports.Parameter =
  exports.Enumeration =
  exports.Bool =
  exports.DateOnly =
  exports.Ipv6 =
  exports.Ipv4 =
  exports.Hostname =
  exports.Uuid =
  exports.Email =
  exports.Regex =
  exports.DateTime =
  exports.Str =
  exports.Int =
  exports.Num =
  exports.Obj =
  exports.Arr =
  exports.BaseParameter =
    void 0
var exceptions_1 = require('./exceptions')
var BaseParameter = /** @class */ (function () {
  function BaseParameter(params) {
    this.isParameter = true
    this.type = null
    this.params = params || {}
    this.generated = true
    if (this.params.required === undefined) this.params.required = true
  }
  BaseParameter.prototype.getValue = function () {
    return {
      type: this.type,
      description: this.params.description,
      example: this.params.example,
      default: this.params.default,
      enum: this.params.enum ? Object.keys(this.params.enum) : undefined,
    }
  }
  BaseParameter.prototype.validate = function (value) {
    if (this.params.enum) {
      if (typeof value !== 'string') {
        throw new exceptions_1.ValidationError('is not one of available options')
      }
      if (this.params.enumCaseSensitive !== false) {
        value = this.params.enum[value]
      } else {
        var key = Object.keys(this.params.enum).find(function (key) {
          return key.toLowerCase() === value.toLowerCase()
        })
        value = this.params.enum[key]
      }
      if (value === undefined) {
        throw new exceptions_1.ValidationError('is not one of available options')
      }
    }
    return value
  }
  BaseParameter.isParameter = true
  return BaseParameter
})()
exports.BaseParameter = BaseParameter
var Arr = /** @class */ (function (_super) {
  __extends(Arr, _super)
  function Arr(innerType, params) {
    var _this = _super.call(this, params) || this
    _this.isArr = true
    _this.innerType = innerType
    return _this
  }
  Arr.prototype.validate = function (value) {
    var _this = this
    value = _super.prototype.validate.call(this, value)
    if (Array.isArray(value)) {
      value = value.map(function (val) {
        return _this.innerType.validate(val)
      })
    } else {
      value = [this.innerType.validate(value)]
    }
    return value
  }
  // @ts-ignore
  Arr.prototype.getValue = function () {
    return {
      type: 'array',
      items: this.innerType.getValue(),
    }
  }
  return Arr
})(BaseParameter)
exports.Arr = Arr
var Obj = /** @class */ (function (_super) {
  __extends(Obj, _super)
  function Obj(fields, params) {
    var _this =
      _super.call(this, params) || // TODO: fix obj params
      this
    _this.isObj = true
    _this.fields = fields
    return _this
  }
  Obj.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    for (var _i = 0, _a = Object.entries(this.fields); _i < _a.length; _i++) {
      var _b = _a[_i],
        key = _b[0],
        param = _b[1]
      try {
        value[key] = param.validate(value[key])
      } catch (e) {
        // @ts-ignore
        e.key = (e.key || '') + ('.' + key)
        throw e
      }
    }
    return value
  }
  // @ts-ignore
  Obj.prototype.getValue = function () {
    var _a
    var result = {
      type: 'object',
      properties: {},
    }
    var required = []
    for (var _i = 0, _b = Object.entries(this.fields); _i < _b.length; _i++) {
      var _c = _b[_i],
        key = _c[0],
        value = _c[1]
      if (((_a = value.params) === null || _a === void 0 ? void 0 : _a.required) === true) {
        required.push(key)
      }
      if (value.getValue) {
        result.properties[key] = value.getValue()
      } else {
        result.properties[key] = value
      }
    }
    if (required.length > 0) {
      result['required'] = required
    }
    return result
  }
  return Obj
})(BaseParameter)
exports.Obj = Obj
var Num = /** @class */ (function (_super) {
  __extends(Num, _super)
  function Num() {
    var _this = (_super !== null && _super.apply(this, arguments)) || this
    _this.type = 'number'
    return _this
  }
  Num.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    value = Number.parseFloat(value)
    if (isNaN(value)) {
      throw new exceptions_1.ValidationError('is not a valid number')
    }
    return value
  }
  return Num
})(BaseParameter)
exports.Num = Num
var Int = /** @class */ (function (_super) {
  __extends(Int, _super)
  function Int() {
    var _this = (_super !== null && _super.apply(this, arguments)) || this
    _this.type = 'integer'
    return _this
  }
  Int.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    value = Number.parseInt(value)
    if (isNaN(value)) {
      throw new exceptions_1.ValidationError('is not a valid integer')
    }
    return value
  }
  return Int
})(Num)
exports.Int = Int
var Str = /** @class */ (function (_super) {
  __extends(Str, _super)
  function Str(params) {
    var _this = _super.call(this, params) || this
    _this.type = 'string'
    return _this
  }
  Str.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    if (this.params.format) {
      if (this.params.format === 'date-time') {
        value = new Date(value)
        if (isNaN(value.getDay())) {
          throw new exceptions_1.ValidationError('is not a valid date time')
        }
      } else if (this.params.format === 'date') {
        if (!value.match(/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/)) {
          throw new exceptions_1.ValidationError('is not a valid date')
        }
        value = new Date(value)
        if (isNaN(value.getDay())) {
          throw new exceptions_1.ValidationError('is not a valid date')
        }
      }
      // TODO: validate remaining formats
    }
    return value
  }
  Str.prototype.getValue = function () {
    return __assign(__assign({}, _super.prototype.getValue.call(this)), { format: this.params.format })
  }
  return Str
})(BaseParameter)
exports.Str = Str
var DateTime = /** @class */ (function (_super) {
  __extends(DateTime, _super)
  function DateTime(params) {
    var _this =
      _super.call(this, __assign(__assign({ example: '2022-09-15T00:00:00Z' }, params), { format: 'date-time' })) ||
      this
    _this.type = 'string'
    return _this
  }
  return DateTime
})(Str)
exports.DateTime = DateTime
var Regex = /** @class */ (function (_super) {
  __extends(Regex, _super)
  function Regex(params) {
    var _this = _super.call(this, params) || this
    _this.type = 'string'
    return _this
  }
  Regex.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    if (!value.match(this.params.format)) {
      if (this.params.patternError) {
        throw new exceptions_1.ValidationError('is not a valid ' + this.params.patternError)
      }
      throw new exceptions_1.ValidationError('does not match the pattern ' + this.params.format)
    }
    return value
  }
  Regex.prototype.getValue = function () {
    return __assign(__assign({}, _super.prototype.getValue.call(this)), { pattern: this.params.format })
  }
  return Regex
})(Str)
exports.Regex = Regex
var Email = /** @class */ (function (_super) {
  __extends(Email, _super)
  function Email(params) {
    var _this =
      _super.call(
        this,
        __assign(__assign({ pattern: '^[^\\s@]+@[^\\s@]+\\.[^\\s@]+$', patternError: 'email' }, params), {
          format: 'email',
        })
      ) || this
    _this.type = 'string'
    return _this
  }
  return Email
})(Regex)
exports.Email = Email
var Uuid = /** @class */ (function (_super) {
  __extends(Uuid, _super)
  function Uuid(params) {
    var _this =
      _super.call(
        this,
        __assign(
          __assign(
            {
              pattern: '^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$',
              patternError: 'uuid',
            },
            params
          ),
          { format: 'uuid' }
        )
      ) || this
    _this.type = 'string'
    return _this
  }
  return Uuid
})(Regex)
exports.Uuid = Uuid
var Hostname = /** @class */ (function (_super) {
  __extends(Hostname, _super)
  function Hostname(params) {
    var _this =
      _super.call(
        this,
        __assign(
          __assign(
            {
              pattern:
                '^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\\-]*[a-zA-Z0-9])\\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\\-]*[A-Za-z0-9])$',
              patternError: 'hostname',
            },
            params
          ),
          { format: 'hostname' }
        )
      ) || this
    _this.type = 'string'
    return _this
  }
  return Hostname
})(Regex)
exports.Hostname = Hostname
var Ipv4 = /** @class */ (function (_super) {
  __extends(Ipv4, _super)
  function Ipv4(params) {
    var _this =
      _super.call(
        this,
        __assign(
          __assign(
            {
              pattern:
                '^(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}$',
              patternError: 'ipv4',
            },
            params
          ),
          { format: 'ipv4' }
        )
      ) || this
    _this.type = 'string'
    return _this
  }
  return Ipv4
})(Regex)
exports.Ipv4 = Ipv4
var Ipv6 = /** @class */ (function (_super) {
  __extends(Ipv6, _super)
  function Ipv6(params) {
    var _this =
      _super.call(
        this,
        __assign(
          __assign(
            {
              pattern:
                '^(?:(?:[a-fA-F\\d]{1,4}:){7}(?:[a-fA-F\\d]{1,4}|:)|(?:[a-fA-F\\d]{1,4}:){6}(?:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|:[a-fA-F\\d]{1,4}|:)|(?:[a-fA-F\\d]{1,4}:){5}(?::(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,2}|:)|(?:[a-fA-F\\d]{1,4}:){4}(?:(?::[a-fA-F\\d]{1,4}){0,1}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,3}|:)|(?:[a-fA-F\\d]{1,4}:){3}(?:(?::[a-fA-F\\d]{1,4}){0,2}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,4}|:)|(?:[a-fA-F\\d]{1,4}:){2}(?:(?::[a-fA-F\\d]{1,4}){0,3}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,5}|:)|(?:[a-fA-F\\d]{1,4}:){1}(?:(?::[a-fA-F\\d]{1,4}){0,4}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,6}|:)|(?::(?:(?::[a-fA-F\\d]{1,4}){0,5}:(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}|(?::[a-fA-F\\d]{1,4}){1,7}|:)))(?:%[0-9a-zA-Z]{1,})?$',
              patternError: 'ipv6',
            },
            params
          ),
          { format: 'ipv6' }
        )
      ) || this
    _this.type = 'string'
    return _this
  }
  return Ipv6
})(Regex)
exports.Ipv6 = Ipv6
var DateOnly = /** @class */ (function (_super) {
  __extends(DateOnly, _super)
  function DateOnly(params) {
    var _this = _super.call(this, __assign(__assign({ example: '2022-09-15' }, params), { format: 'date' })) || this
    _this.type = 'string'
    return _this
  }
  return DateOnly
})(Str)
exports.DateOnly = DateOnly
var Bool = /** @class */ (function (_super) {
  __extends(Bool, _super)
  function Bool() {
    var _this = (_super !== null && _super.apply(this, arguments)) || this
    _this.type = 'boolean'
    _this.validValues = ['true', 'false']
    return _this
  }
  Bool.prototype.validate = function (value) {
    value = _super.prototype.validate.call(this, value)
    value = value.toLowerCase()
    if (!this.validValues.includes(value)) {
      throw new exceptions_1.ValidationError('is not a valid boolean, allowed values true or false')
    }
    value = value === 'true'
    return value
  }
  return Bool
})(BaseParameter)
exports.Bool = Bool
var Enumeration = /** @class */ (function (_super) {
  __extends(Enumeration, _super)
  function Enumeration(values, params) {
    var _this = _super.call(this, __assign(__assign({}, params || {}), { enum: values })) || this
    _this.isEnum = true
    _this.keys = Object.keys(values)
    _this.values = values
    return _this
  }
  return Enumeration
})(Str)
exports.Enumeration = Enumeration
var Parameter = /** @class */ (function () {
  function Parameter(location, rawType, params) {
    this.location = location
    this.rawType = rawType
    if (params.required === undefined) params.required = true
    this.params = params
    this.type = this.getType(rawType, params)
  }
  Parameter.prototype.getType = function (type, params) {
    if (type.generated === true) {
      return type
    }
    if (type.isParameter === true) {
      // @ts-ignore
      return new type(__assign({}, params))
    }
    if (type === String) {
      return new Str(__assign({}, params))
    }
    if (type === Number) {
      return new Num(__assign({}, params))
    }
    if (type === Boolean) {
      return new Bool(__assign({}, params))
    }
    if (type === Date) {
      return new DateTime()
    }
    if (Array.isArray(type)) {
      if (type.length === 0) {
        throw new Error('Arr must have a type')
      }
      return new Arr(this.getType(type[0], params), __assign({}, params))
    }
    if (typeof type === 'object') {
      var parsed = {}
      for (var _i = 0, _a = Object.entries(type); _i < _a.length; _i++) {
        var _b = _a[_i],
          key = _b[0],
          value = _b[1]
        parsed[key] = this.getType(value, {})
      }
      return new Obj(parsed, params)
    }
    throw new Error(type + ' not implemented')
  }
  Parameter.prototype.getValue = function () {
    var schema = removeUndefinedFields(this.type.getValue())
    return {
      description: this.params.description,
      required: this.params.required,
      schema: schema,
      name: this.params.name,
      in: this.location,
    }
  }
  Parameter.prototype.validate = function (value) {
    if (value === undefined || value === null) {
      if (this.params.default !== undefined && this.params.default !== null) {
        value = this.params.default
      } else {
        if (this.params.required) {
          throw new exceptions_1.ValidationError('is required')
        } else {
          return null
        }
      }
    }
    value = this.type.validate(value)
    return value
  }
  return Parameter
})()
exports.Parameter = Parameter
var Body = /** @class */ (function (_super) {
  __extends(Body, _super)
  function Body(rawType, params) {
    var _this = _super.call(this, null, rawType, {}) || this
    _this.paramsBody = params
    return _this
  }
  Body.prototype.getValue = function () {
    var _a, _b
    var schema = removeUndefinedFields(this.type.getValue())
    var param = {
      description: (_a = this.paramsBody) === null || _a === void 0 ? void 0 : _a.description,
      content: {},
    }
    param.content[((_b = this.paramsBody) === null || _b === void 0 ? void 0 : _b.contentType) || 'application/json'] =
      { schema: schema }
    return param
  }
  return Body
})(Parameter)
exports.Body = Body
var Resp = /** @class */ (function (_super) {
  __extends(Resp, _super)
  function Resp(rawType, params) {
    return _super.call(this, null, rawType, params) || this
  }
  // @ts-ignore
  Resp.prototype.getValue = function () {
    var _a
    var value = _super.prototype.getValue.call(this)
    var contentType = ((_a = this.params) === null || _a === void 0 ? void 0 : _a.contentType)
      ? 'this.params?.contentType'
      : 'application/json'
    var param = {
      description: this.params.description || 'Successful Response',
      content: {},
    }
    param.content[contentType] = { schema: value.schema }
    return param
  }
  return Resp
})(Parameter)
exports.Resp = Resp
function Query(type, params) {
  if (params === void 0) {
    params = {}
  }
  return new Parameter('query', type, params)
}
exports.Query = Query
function Path(type, params) {
  if (params === void 0) {
    params = {}
  }
  return new Parameter('path', type, params)
}
exports.Path = Path
function Header(type, params) {
  if (params === void 0) {
    params = {}
  }
  return new Parameter('header', type, params)
}
exports.Header = Header
function Cookie(type, params) {
  if (params === void 0) {
    params = {}
  }
  return new Parameter('cookie', type, params)
}
exports.Cookie = Cookie
function extractParameter(request, query, name, location) {
  if (location === 'query') {
    return query[name]
  }
  if (location === 'path') {
    return request.params[name]
  }
  if (location === 'header') {
    return request.headers.get(name)
  }
  if (location === 'cookie') {
    throw new Error('Cookie parameters not implemented yet')
  }
}
exports.extractParameter = extractParameter
function extractQueryParameters(request) {
  var url = decodeURIComponent(request.url).split('?')
  if (url.length === 1) {
    return {}
  }
  var query = url[1]
  var params = {}
  for (var _i = 0, _a = query.split('&'); _i < _a.length; _i++) {
    var param = _a[_i]
    var paramSplitted = param.split('=')
    var key = paramSplitted[0]
    var value = paramSplitted[1]
    if (params[key] === undefined) {
      params[key] = value
    } else if (!Array.isArray(params[key])) {
      params[key] = [params[key], value]
    } else {
      params[key].push(value)
    }
  }
  return params
}
exports.extractQueryParameters = extractQueryParameters
function Required(param) {
  param.params.required = true
  return param
}
exports.Required = Required
function removeUndefinedFields(obj) {
  for (var _i = 0, _a = Object.entries(obj); _i < _a.length; _i++) {
    var _b = _a[_i],
      key = _b[0],
      value = _b[1]
    if (typeof value === 'object' && !Array.isArray(value)) obj[key] = removeUndefinedFields(value)
    if (value === undefined) {
      delete obj[key]
    }
  }
  return obj
}
exports.removeUndefinedFields = removeUndefinedFields
function getFormatedParameters(params) {
  var formated = []
  var isArray = Array.isArray(params)
  for (var _i = 0, _a = Object.entries(params || {}); _i < _a.length; _i++) {
    var _b = _a[_i],
      key = _b[0],
      parameter = _b[1]
    if (isArray && !parameter.params.name) {
      throw new Error('Parameter must have a defined name when using as Array')
    }
    var name = parameter.params.name ? parameter.params.name : key
    formated.push(__assign(__assign({}, parameter.getValue()), { name: name }))
  }
  return formated
}
exports.getFormatedParameters = getFormatedParameters
